"use strict";

var Functions = {};

/**
 * Bounds value between opt_min and opt_max (result will be not smaller than
 * opt_min and not bigger than opt_max).
 */
Functions.bound = function(value, minVal, maxVal) {
  if (minVal !== null && minVal !== undefined) {
    value = Math.max(value, minVal);
  }
  if (maxVal !== null && maxVal !== undefined) {
    value = Math.min(value, maxVal);
  }
  return value;
};

Functions.degreesToRadians = function(deg) {
  return deg * (Math.PI / 180);
};

Functions.radiansToDegrees = function(rad) {
  return rad / (Math.PI / 180);
};

Functions.intToColorString = function(value) {
  /* jslint bitwise: true */
  var timmedValue = (value & 0xFFFFFF);
  var colorStr = timmedValue.toString(16);
  while (colorStr.length < 6) {
    colorStr = "0" + colorStr;
  }
  return '#' + colorStr;
};

/**
 * Returns stack trace.
 * 
 * @returns stack trace
 */
Functions.stackTrace = function() {
  var err = new Error();
  return err.stack;
};

/**
 * Returns the position of the element on html page.
 * 
 * @param element
 *          element for which we want to get the position (top left corner)
 * 
 * @return coordinates of the element
 * 
 */
Functions.getPosition = function(element) {
  var xPosition = 0;
  var yPosition = 0;

  while (element) {
    xPosition += (element.offsetLeft - element.scrollLeft + element.clientLeft);
    yPosition += (element.offsetTop - element.scrollTop + element.clientTop);
    element = element.offsetParent;
  }
  return {
    x : xPosition,
    y : yPosition
  };
};

/**
 * Checks if the point given as a first argument belongs to a polygon defined as
 * a second parameter. Both: point and polygon should use google.map.point
 * class.
 * 
 * @param point
 *          point which we want to check
 * 
 * @param polygon
 *          polygon where we check the point
 */

Functions.pointInsidePolygon = function(point, polygon) {
  var x = point.x;
  var y = point.y;

  var inside = false;
  for (var i = 0, j = polygon.length - 1; i < polygon.length; j = i++) {
    var xi = polygon[i].x, yi = polygon[i].y;
    var xj = polygon[j].x, yj = polygon[j].y;
    var intersect = ((yi > y) !== (yj > y)) && (x < (xj - xi) * (y - yi) / (yj - yi) + xi);
    if (intersect) {
      inside = !inside;
    }
  }
  return inside;
};

/**
 * In a browser variable we store inforamtion about browser user is currently
 * using. Right now only IE is suppoerted.
 */
Functions.browser = {
  init : function() {

    this.name = "Unknown";
    this.version = "Unknown";

    if (typeof navigator !== 'undefined') {
      // Get the user agent string
      var ua = navigator.userAgent;
      this.compatibilityMode = false;
      var re;
      if (navigator.appName === 'Microsoft Internet Explorer') {
        this.name = "IE";
        re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
        if (re.exec(ua) !== undefined && re.exec(ua) !== null) {
          this.version = parseFloat(RegExp.$1);
        }
        if (ua.indexOf("MSIE 7.0") > -1) {
          this.compatibilityMode = true;
        }
      } else if (navigator.appName === 'Netscape') {
        this.name = "Other";
        ua = navigator.userAgent;
        re = new RegExp("Trident/.*rv[ :]*([0-9]{1,}[\.0-9]{0,})");
        if (re.exec(ua) !== undefined && re.exec(ua) !== null) {
          this.version = parseFloat(RegExp.$1);
        }
      }
    }
  }
};

Functions.browser.init();

/**
 * Returns true if parameter is integer, false otherwise.
 * 
 * @param n
 *          object to check
 */
Functions.isInt = function(n) {
  return Number(n) === n && n % 1 === 0;
};

/**
 * Returns true if parameter is a DOM element, false otherwise.
 * 
 * @param o
 *          object to check
 */
Functions.isDomElement = function(o) {
  if (!o) {
    return false;
  }
  return (typeof HTMLElement === "object" ? o instanceof HTMLElement : // DOM2
  o && typeof o === "object" && o !== null && o.nodeType === 1 && typeof o.nodeName === "string");
};

Functions.overlayToColor = function(elementOverlay) {
  /* jslint bitwise: true */
  if (elementOverlay === null || elementOverlay === undefined) {
    throw new Error("elementOverlay cannot be null!");
  } else if (elementOverlay.color !== undefined && elementOverlay.color !== null) {
    return this.intToColorString(elementOverlay.color.value);
  } else if (elementOverlay.value !== undefined && elementOverlay.value !== null) {
    var ratio = 0;
    var color;
    if (elementOverlay.value < 0) {
      ratio = -elementOverlay.value;
      color = ServerConnector.getMinOverlayColorInt();
    } else {
      ratio = elementOverlay.value;
      color = ServerConnector.getMaxOverlayColorInt();
    }
    var red = color & 0xFF0000;
    red = red * ratio;
    red = parseInt(red);
    red = red & 0xFF0000;

    var green = color & 0x00FF00;
    green = green * ratio;
    green = parseInt(green);
    green = green & 0x00FF00;

    var blue = color & 0x0000FF;
    blue = blue * ratio;
    blue = parseInt(blue);
    blue = blue & 0x0000FF;

    color = red | green | blue;
    return this.intToColorString(color);
  } else {
    throw new Error("elementOverlay doesn't have neither color nor value set!");
  }
};

module.exports = Functions;
