"use strict";

require("./mocha-config.js");

var Alias = require('../../main/js/map/data/Alias');
var Project = require('../../main/js/map/data/Project');
var Reaction = require('../../main/js/map/data/Reaction');
var ServerConnector = require('../../main/js/ServerConnector');

var logger = require('./logger');
var chai = require('chai');
var assert = chai.assert;

describe('ServerConnector', function() {
  it('getProject', function() {
    return ServerConnector.getProject().then(function(result) {
      assert.ok(result instanceof Project);
      assert.equal(result.getProjectId(), "sample");
      assert.equal(logger.getWarnings().length, 0);
    });
  });

  it('getReactions with empty list of ids', function() {
    return ServerConnector.getReactions([]).then(function(result) {
      assert.equal(result.length, 2);
      var reaction = result[0];
      assert.ok(reaction instanceof Reaction);
      assert.equal(reaction.getId(), 153513);
      assert.equal(reaction.getModelId(), 15781);
    });
  });

  it('getReactions without ids', function() {
    return ServerConnector.getReactions([]).then(function(result) {
      assert.equal(result.length, 2);
      var reaction = result[0];
      assert.ok(reaction instanceof Reaction);
      assert.equal(reaction.getId(), 153513);
      assert.equal(reaction.getModelId(), 15781);
    });
  });

  it('getElements with empty list of ids', function() {
    return ServerConnector.getLightAliases([]).then(function(result) {
      assert.equal(result.length, 2);
      var reaction = result[0];
      assert.ok(reaction instanceof Alias);
      assert.equal(reaction.getId(), 329176);
      assert.equal(reaction.getModelId(), 15781);
    });
  });

});
